/******************************************************************************************
 *
 * CHECK_USERS.C
 *
 * Program: Current users plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-22-1999
 *
 * Command line: CHECK_USERS <wusers> <cusers>
 *
 * Description:
 *
 * This plugin will use the /usr/bin/who command to check the number of users currently logged
 * into the system.  If number of logged in user exceeds the number specified by the
 * <cusers> option, a STATE_CRITICAL is return.  It it exceeds <wusers>, a STATE_WARNING
 * is returned.  Errors reading the output from the who command result in a STATE_UNKNOWN
 * error.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256

int main(int argc, char **argv){
  /*	char command_line[256];*/
	int wusers;
	int cusers;
	int users=0;
	int result;
	int found;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char total_users[32];
	char *temp_buffer;

	if(argc!=3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Current users plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-22-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wusers> <cusers>\n",argv[0]);
		printf("\n");
		printf("Note:\n");
		printf(" <wusers> = Number of logged in users necessary to result in a WARNING state\n");
		printf(" <cusers> = Number of logged in users necessary to result in a CRITICAL state\n");
		printf("\n");
		printf("This plugin will check the number of users currently logged in on the local\n");
		printf("system and generate an error if the number exceeds the thresholds specified.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the number of logged in users to use as warning and critical thresholds */
	wusers=atoi(argv[1]);
	cusers=atoi(argv[2]);

	result=STATE_OK;

	/* open the temp file */
	fp=popen(WHO_COMMAND,"r");
	if(fp==NULL){
		printf("Could not open pipe: %s\n",WHO_COMMAND);
		return STATE_UNKNOWN;
	        }

	found=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		/* skip all lines except the total user count */
		if(input_buffer[0]!='#')
			continue;

		/* get total logged in users */
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"=");
		if(temp_buffer!=NULL){
			strcpy(total_users,temp_buffer);
			found=1;
			}
		else{
			result=STATE_UNKNOWN;
			break;
			}
	        }

	/* close the temp file */
	pclose(fp);

	/* if we couldn't find the user count, return an UNKNOWN state */
	if(!found)
		result=STATE_UNKNOWN;

	/* else check the user count against warning and critical thresholds */
	else if(result!=STATE_UNKNOWN){
		users=atoi(total_users);
		if(users>=cusers)
			result=STATE_CRITICAL;
		else if(users>=wusers)
			result=STATE_WARNING;
	        }
	
	if(result==STATE_OK)
		printf("Users ok - %d users logged in\n",users);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n");
	else{
		printf("%d users currently logged in\n",users);
	        }

	return result;
        }
